@@ -3,8 +3,8 @@
 	Windows Forms example.
 
  Copyright (c) 2007 by Educational Technology Resources, Inc.  Others 
- may also hold copyrights on code in this file.  See the CREDITS file in
- the top directory of the distribution for details.
+ may also hold copyrights on code in this file.  See the CREDITS.txt
+ file in the top directory of the distribution for details.
 
  This file is part of MySQL++.
 
@@ -103,9 +103,8 @@ namespace wforms {
 
 			if (res) {
 				// Display the result set
-				mysqlpp::Row row;
-				for (mysqlpp::Row::size_type i = 0; row = res.at(i); ++i) {
-					AddMessage(ToUCS2(row.at(0)));
+				for (size_t i = 0; i < res.num_rows(); ++i) {
+					AddMessage(ToUCS2(res[i][0]));
 				}
 
 				// Retreive was successful, so save user inputs now
