@@ -1,7 +1,7 @@
 /***********************************************************************
  dbdriver.cpp - Implements the DBDriver class.
 
- Copyright (c) 2005-2007 by Educational Technology Resources, Inc.
+ Copyright © 2005-2009, 2018 by Educational Technology Resources, Inc.
  Others may also hold copyrights on code in this file.  See the
  CREDITS.txt file in the top directory of the distribution for details.
 
@@ -28,15 +28,9 @@
 
 #include "exceptions.h"
 
-#include <sstream>
+#include <cstring>
 #include <memory>
-
-// An argument was added to mysql_shutdown() in MySQL 4.1.3 and 5.0.1.
-#if ((MYSQL_VERSION_ID >= 40103) && (MYSQL_VERSION_ID <= 49999)) || (MYSQL_VERSION_ID >= 50001)
-#	define SHUTDOWN_ARG ,SHUTDOWN_DEFAULT
-#else
-#	define SHUTDOWN_ARG
-#endif
+#include <sstream>
 
 using namespace std;
 
@@ -45,7 +39,10 @@ namespace mysqlpp {
 DBDriver::DBDriver() :
 is_connected_(false)
 {
-	mysql_init(&mysql_);
+	// We won't allow calls to mysql_*() functions that take a MYSQL
+	// object until we get a connection up.  Such calls are nonsense.
+	// MySQL++ coped with them before, but this masks bugs.
+	memset(&mysql_, 0, sizeof(mysql_));
 }
 
 
@@ -74,59 +71,74 @@ DBDriver::connect(const char* host, const char* socket_name,
 		unsigned int port, const char* db, const char* user,
 		const char* password)
 {
-	// Drop previous connection, if any
-	if (connected()) {
-		disconnect();
-	}
-
-	// Set defaults for connection options.  User can override these
-	// by calling set_option() before connect().
-	set_option_default(new ReadDefaultFileOption("my"));
-
-	// Establish the connection
 	return is_connected_ =
+			connect_prepare() &&
 			mysql_real_connect(&mysql_, host, user, password, db,
-			port, socket_name, mysql_.client_flag);
+				port, socket_name, mysql_.client_flag);
 }
 
 
 bool
 DBDriver::connect(const MYSQL& other)
 {
-	// Drop previous connection, if any
+	return is_connected_ =
+			connect_prepare() &&
+			mysql_real_connect(&mysql_, other.host, other.user,
+				other.passwd, other.db, other.port, other.unix_socket,
+				other.client_flag);
+}
+
+
+bool
+DBDriver::connect_prepare()
+{
+	// Drop previous connection, if any, then prepare underlying C API
+	// library to establish a new connection.
 	if (connected()) {
 		disconnect();
 	}
 
-	// Set defaults for connection options.  User can override these
-	// by calling set_option() before connect().
-	set_option_default(new ReadDefaultFileOption("my"));
+	// Set up to call MySQL C API
+	mysql_init(&mysql_);
 
-	// Establish the connection
-	return is_connected_ =
-			mysql_real_connect(&mysql_, other.host, other.user,
-			other.passwd, other.db, other.port, other.unix_socket,
-			other.client_flag);
+    // Apply any pending options
+	error_message_.clear();
+	OptionListIt it = pending_options_.begin();
+	while (it != pending_options_.end() && set_option_impl(*it)) {
+		++it;
+	}
+	if (it == pending_options_.end()) {
+		pending_options_.clear();
+		return true;
+	}
+	else {
+		return false;
+	}
 }
 
 
 void
 DBDriver::copy(const DBDriver& other)
 {
+	if (connected()) {
+		disconnect();
+	}
+
 	if (other.connected()) {
 		connect(other.mysql_);
 	}
-	else {
-		is_connected_ = false;
-	}
 }
 
 
 void
 DBDriver::disconnect()
 {
-	mysql_close(&mysql_);
-	is_connected_ = false;
+	if (is_connected_) {
+		mysql_close(&mysql_);
+		memset(&mysql_, 0, sizeof(mysql_));
+		is_connected_ = false;
+		error_message_.clear();
+	}
 }
 
 
@@ -134,14 +146,82 @@ bool
 DBDriver::enable_ssl(const char* key, const char* cert,
 		const char* ca, const char* capath, const char* cipher)
 {
+	error_message_.clear();
 #if defined(HAVE_MYSQL_SSL_SET)
 	return mysql_ssl_set(&mysql_, key, cert, ca, capath, cipher) == 0;
 #else
+	(void)key;
+	(void)cert;
+	(void)ca;
+	(void)capath;
+	(void)cipher;
 	return false;
 #endif
 }
 
 
+size_t
+DBDriver::escape_string(std::string* ps, const char* original,
+		size_t length)
+{
+	error_message_.clear();
+
+	if (ps == 0) {
+		// Can't do any real work!
+		return 0;
+	}
+	else if (original == 0) {
+		// ps must point to the original data as well as to the
+		// receiving string, so get the pointer and the length from it.
+		original = ps->data();
+		length = ps->length();
+	}
+	else if (length == 0) {
+		// We got a pointer to a C++ string just for holding the result
+		// and also a C string pointing to the original, so find the
+		// length of the original.
+		length = strlen(original);
+	}
+
+	char* escaped = new char[length * 2 + 1];
+	length = escape_string(escaped, original, length);
+	ps->assign(escaped, length);
+	delete[] escaped;
+
+	return length;
+}
+
+
+size_t
+DBDriver::escape_string_no_conn(std::string* ps, const char* original,
+		size_t length)
+{
+	if (ps == 0) {
+		// Can't do any real work!
+		return 0;
+	}
+	else if (original == 0) {
+		// ps must point to the original data as well as to the
+		// receiving string, so get the pointer and the length from it.
+		original = ps->data();
+		length = ps->length();
+	}
+	else if (length == 0) {
+		// We got a pointer to a C++ string just for holding the result
+		// and also a C string pointing to the original, so find the
+		// length of the original.
+		length = strlen(original);
+	}
+
+	char* escaped = new char[length * 2 + 1];
+	length = DBDriver::escape_string_no_conn(escaped, original, length);
+	ps->assign(escaped, length);
+	delete[] escaped;
+
+	return length;
+}
+
+
 DBDriver&
 DBDriver::operator=(const DBDriver& rhs)
 {
@@ -153,6 +233,7 @@ DBDriver::operator=(const DBDriver& rhs)
 string
 DBDriver::query_info()
 {
+	error_message_.clear();
 	const char* i = mysql_info(&mysql_);
 	return i ? string(i) : string();
 }
@@ -169,7 +250,11 @@ DBDriver::set_option(unsigned int o, bool arg)
 	}
 	
 	if ((n == 1) &&
+#ifdef CLIENT_LONG_PASSWORD
 			(o >= CLIENT_LONG_PASSWORD) &&
+#else
+			(o >= CLIENT_MYSQL) &&
+#endif
 #if MYSQL_VERSION_ID > 40000	// highest flag value varies by version
 			(o <= CLIENT_MULTI_RESULTS)
 #else
@@ -194,11 +279,25 @@ DBDriver::set_option(unsigned int o, bool arg)
 }
 
 
-std::string
+bool
 DBDriver::set_option(Option* o)
+{
+	if (connected()) {
+		return set_option_impl(o);
+	}
+	else {
+		error_message_.clear();
+        pending_options_.push_back(o);
+		return true;  // we won't know if it fails until ::connect()
+	}
+}
+
+
+bool
+DBDriver::set_option_impl(Option* o)
 {
 	std::ostringstream os;
-	std::auto_ptr<Option> cleanup(o);
+	UNIQUE_PTR(Option) cleanup(o);
 
 	switch (o->set(this)) {
 		case Option::err_NONE:
@@ -218,16 +317,34 @@ DBDriver::set_option(Option* o)
 		case Option::err_connected:
 			os << "Option can only be set before connection is established";
 			break;
+
+		case Option::err_disconnected:
+			os << "Option can only be set while the connection is established";
+			break;
 	}
 
-	return os.str();
+	error_message_ = os.str();
+	return error_message_.empty();
 }
 
 
 bool
 DBDriver::shutdown()
 {
-	return mysql_shutdown(&mysql_ SHUTDOWN_ARG);
+	error_message_.clear();
+#if (MYSQL_VERSION_ID >= 80000)
+    // The C API function we were depending on is removed in 8.0.0.  It
+    // was replaced in 8.0.1, but they claim they're going to remove it
+    // again later, so we prefer to behave as if it's gone for good as
+    // of 8.0.0, so we've got a well-tested workaround before then.
+    return connected() ? execute("SHUTDOWN", 8) : false;
+#elif ((MYSQL_VERSION_ID >= 40103) && (MYSQL_VERSION_ID <= 49999)) || (MYSQL_VERSION_ID >= 50001)
+    // An argument was added to mysql_shutdown() in MySQL 4.1.3 and 5.0.1.
+	return mysql_shutdown(&mysql_, SHUTDOWN_DEFAULT);
+#else
+    // Prior versions had no argument
+	return mysql_shutdown(&mysql_);
+#endif
 }
 
 
