@@ -1,8 +1,8 @@
 /***********************************************************************
  stadapter.cpp - Implements the SQLTypeAdapter class.
 
- Copyright (c) 1998 by Kevin Atkinson, (c) 1999-2001 by MySQL AB, and
- (c) 2004-2008 by Educational Technology Resources, Inc.  Others may
+ Copyright © 1998 by Kevin Atkinson, © 1999-2001 by MySQL AB, and
+ © 2004-2009, 2018 by Educational Technology Resources, Inc.  Others may
  also hold copyrights on code in this file.  See the CREDITS.txt file
  in the top directory of the distribution for details.
 
@@ -31,6 +31,7 @@
 #include "stream2string.h"
 
 #include <iomanip>
+#include <limits>
 #include <sstream>
 
 using namespace std;
@@ -105,14 +106,14 @@ is_processed_(false)
 }
 #endif
 
-SQLTypeAdapter::SQLTypeAdapter(sql_tinyint i) :
+SQLTypeAdapter::SQLTypeAdapter(tiny_int<signed char> i) :
 buffer_(new SQLBuffer(stream2string(i), typeid(i), false)),
 is_processed_(false)
 {
 }
 
 #if !defined(DOXYGEN_IGNORE)
-SQLTypeAdapter::SQLTypeAdapter(Null<sql_tinyint> i) :
+SQLTypeAdapter::SQLTypeAdapter(Null<tiny_int<signed char> > i) :
 buffer_(new SQLBuffer(i.is_null ? null_str : stream2string(i),
 		i.is_null ? typeid(void) : typeid(i.data), i.is_null)),
 is_processed_(false)
@@ -120,14 +121,14 @@ is_processed_(false)
 }
 #endif
 
-SQLTypeAdapter::SQLTypeAdapter(sql_tinyint_unsigned i) :
+SQLTypeAdapter::SQLTypeAdapter(tiny_int<unsigned char> i) :
 buffer_(new SQLBuffer(stream2string(i), typeid(i), false)),
 is_processed_(false)
 {
 }
 
 #if !defined(DOXYGEN_IGNORE)
-SQLTypeAdapter::SQLTypeAdapter(Null<sql_tinyint_unsigned> i) :
+SQLTypeAdapter::SQLTypeAdapter(Null<tiny_int<unsigned char> > i) :
 buffer_(new SQLBuffer(i.is_null ? null_str : stream2string(i),
 		i.is_null ? typeid(void) : typeid(i.data), i.is_null)),
 is_processed_(false)
@@ -258,17 +259,32 @@ is_processed_(false)
 SQLTypeAdapter::SQLTypeAdapter(float f) :
 is_processed_(false)
 {
-	ostringstream outs;
-	outs.precision(9);	// max dec digits needed for IEEE 754 32-bit float
-	outs << f;
-	buffer_ = new SQLBuffer(outs.str(), typeid(f), false);
+	typedef numeric_limits<float> nlf;
+	if ((nlf::has_infinity && (f == nlf::infinity())) ||
+			(nlf::has_quiet_NaN && (f == nlf::quiet_NaN())) ||
+			(nlf::has_signaling_NaN && (f == nlf::signaling_NaN()))) {
+		// f isn't null-able, but it's infinite or NaN, so store it
+		// as a 0.  This at least prevents syntactically-invalid SQL.
+		buffer_ = new SQLBuffer("0", typeid(f), true);
+	}
+	else {
+		ostringstream outs;
+		outs.precision(9);	// max dec digits needed for IEEE 754 32-bit float
+		outs << f;
+		buffer_ = new SQLBuffer(outs.str(), typeid(f), false);
+	}
 }
 
 #if !defined(DOXYGEN_IGNORE)
 SQLTypeAdapter::SQLTypeAdapter(Null<float> f) :
 is_processed_(false)
 {
-	if (f.is_null) {
+	typedef numeric_limits<float> nlf;
+	if (f.is_null ||
+			(nlf::has_infinity && (f.data == nlf::infinity())) ||
+			(nlf::has_quiet_NaN && (f.data == nlf::quiet_NaN())) ||
+			(nlf::has_signaling_NaN && (f.data == nlf::signaling_NaN()))) {
+		// MySQL wants infinite and NaN FP values stored as SQL NULL
 		buffer_ = new SQLBuffer(null_str, typeid(void), true);
 	}
 	else {
@@ -283,17 +299,32 @@ is_processed_(false)
 SQLTypeAdapter::SQLTypeAdapter(double f) :
 is_processed_(false)
 {
-	ostringstream outs;
-	outs.precision(17);	// max dec digits needed for IEEE 754 64-bit float
-	outs << f;
-	buffer_ = new SQLBuffer(outs.str(), typeid(f), false);
+	typedef numeric_limits<double> nld;
+	if ((nld::has_infinity && (f == nld::infinity())) ||
+			(nld::has_quiet_NaN && (f == nld::quiet_NaN())) ||
+			(nld::has_signaling_NaN && (f == nld::signaling_NaN()))) {
+		// f isn't null-able, but it's infinite or NaN, so store it
+		// as a 0.  This at least prevents syntactically-invalid SQL.
+		buffer_ = new SQLBuffer("0", typeid(f), true);
+	}
+	else {
+		ostringstream outs;
+		outs.precision(17);	// max dec digits needed for IEEE 754 64-bit float
+		outs << f;
+		buffer_ = new SQLBuffer(outs.str(), typeid(f), false);
+	}
 }
 
 #if !defined(DOXYGEN_IGNORE)
 SQLTypeAdapter::SQLTypeAdapter(Null<double> f) :
 is_processed_(false)
 {
-	if (f.is_null) {
+	typedef numeric_limits<double> nld;
+	if (f.is_null ||
+			(nld::has_infinity && (f.data == nld::infinity())) ||
+			(nld::has_quiet_NaN && (f.data == nld::quiet_NaN())) ||
+			(nld::has_signaling_NaN && (f.data == nld::signaling_NaN()))) {
+		// MySQL wants infinite and NaN FP values stored as SQL NULL
 		buffer_ = new SQLBuffer(null_str, typeid(void), true);
 	}
 	else {
@@ -368,7 +399,7 @@ SQLTypeAdapter&
 SQLTypeAdapter::assign(const char* pc, int len)
 {
 	if (len < 0) {
-		len = strlen(pc);
+		len = int(strlen(pc));
 	}
 
 	buffer_ = new SQLBuffer(pc, len, mysql_type_info::string_type, false);
@@ -385,19 +416,19 @@ SQLTypeAdapter::assign(const null_type&)
 }
 
 char
-SQLTypeAdapter::at(size_type i) const throw(std::out_of_range)
+SQLTypeAdapter::at(size_type i) const MAY_THROW(std::out_of_range)
 {
 	if (buffer_) {
 		if (i <= length()) {
 			return *(buffer_->data() + i);
 		}
 		else {
-			throw BadIndex("Not enough chars in SQLTypeAdapter", i,
-					length());
+			throw BadIndex("Not enough chars in SQLTypeAdapter", int(i),
+					int(length()));
 		}
 	}
 	else {
-		throw BadIndex("SQLTypeAdapter buffer not initialized", i, -1);
+		throw BadIndex("SQLTypeAdapter buffer not initialized", int(i), -1);
 	}
 }
 
