@@ -31,7 +31,11 @@
 #include "myset.h"
 #include "sql_types.h"
 
-#include <mysql.h>
+#if defined(MYSQLPP_MYSQL_HEADERS_BURIED)
+#	include <mysql/mysql.h>
+#else
+#	include <mysql.h>
+#endif
 
 #include <string>
 
@@ -80,8 +84,14 @@ const mysql_type_info::sql_type_info mysql_type_info::types[] = {
 			mysql_ti_sql_type_info::tf_unsigned),
 	sql_type_info("FLOAT NOT NULL", typeid(sql_float),
 			MYSQL_TYPE_FLOAT, mysql_ti_sql_type_info::tf_default),
+	sql_type_info("FLOAT UNSIGNED NOT NULL", typeid(sql_float),
+			MYSQL_TYPE_FLOAT, mysql_ti_sql_type_info::tf_default |
+			mysql_ti_sql_type_info::tf_unsigned),
 	sql_type_info("DOUBLE NOT NULL", typeid(sql_double),
 			MYSQL_TYPE_DOUBLE, mysql_ti_sql_type_info::tf_default),
+	sql_type_info("DOUBLE UNSIGNED NOT NULL", typeid(sql_double),
+			MYSQL_TYPE_DOUBLE, mysql_ti_sql_type_info::tf_default |
+			mysql_ti_sql_type_info::tf_unsigned),
 	sql_type_info("NULL NOT NULL", typeid(void),
 			MYSQL_TYPE_NULL, mysql_ti_sql_type_info::tf_default),
 	sql_type_info("TIMESTAMP NOT NULL", typeid(sql_timestamp),
@@ -91,6 +101,11 @@ const mysql_type_info::sql_type_info mysql_type_info::types[] = {
 	sql_type_info("BIGINT UNSIGNED NOT NULL", typeid(sql_bigint_unsigned),
 			MYSQL_TYPE_LONGLONG, mysql_ti_sql_type_info::tf_default |
 			mysql_ti_sql_type_info::tf_unsigned),
+	sql_type_info("BIGINT NOT NULL", typeid(longlong),
+			MYSQL_TYPE_LONGLONG, mysql_ti_sql_type_info::tf_default),
+	sql_type_info("BIGINT UNSIGNED NOT NULL", typeid(ulonglong),
+			MYSQL_TYPE_LONGLONG, mysql_ti_sql_type_info::tf_default |
+			mysql_ti_sql_type_info::tf_unsigned),
 	sql_type_info("MEDIUMINT NOT NULL", typeid(sql_mediumint),
 			MYSQL_TYPE_INT24, mysql_ti_sql_type_info::tf_unsigned),
 	sql_type_info("MEDIUMINT UNSIGNED NOT NULL", typeid(sql_mediumint_unsigned),
@@ -149,9 +164,17 @@ const mysql_type_info::sql_type_info mysql_type_info::types[] = {
 	sql_type_info("FLOAT NULL", typeid(Null<sql_float>),
 			MYSQL_TYPE_FLOAT, mysql_ti_sql_type_info::tf_default |
 			mysql_ti_sql_type_info::tf_null),
+	sql_type_info("FLOAT UNSIGNED NULL", typeid(Null<sql_float>),
+			MYSQL_TYPE_FLOAT, mysql_ti_sql_type_info::tf_default |
+			mysql_ti_sql_type_info::tf_null |
+			mysql_ti_sql_type_info::tf_unsigned),
 	sql_type_info("DOUBLE NULL", typeid(Null<sql_double>),
 			MYSQL_TYPE_DOUBLE, mysql_ti_sql_type_info::tf_default |
 			mysql_ti_sql_type_info::tf_null),
+	sql_type_info("DOUBLE UNSIGNED NULL", typeid(Null<sql_double>),
+			MYSQL_TYPE_DOUBLE, mysql_ti_sql_type_info::tf_default |
+			mysql_ti_sql_type_info::tf_null |
+			mysql_ti_sql_type_info::tf_unsigned),
 	sql_type_info("NULL NULL", typeid(Null<void>),
 			MYSQL_TYPE_NULL, mysql_ti_sql_type_info::tf_null),
 	sql_type_info("TIMESTAMP NULL", typeid(Null<sql_timestamp>),
@@ -163,6 +186,13 @@ const mysql_type_info::sql_type_info mysql_type_info::types[] = {
 			MYSQL_TYPE_LONGLONG, mysql_ti_sql_type_info::tf_default |
 			mysql_ti_sql_type_info::tf_null |
 			mysql_ti_sql_type_info::tf_unsigned),
+	sql_type_info("BIGINT NULL", typeid(Null<longlong>),
+			MYSQL_TYPE_LONGLONG, mysql_ti_sql_type_info::tf_default |
+			mysql_ti_sql_type_info::tf_null),
+	sql_type_info("BIGINT UNSIGNED NULL", typeid(Null<ulonglong>),
+			MYSQL_TYPE_LONGLONG, mysql_ti_sql_type_info::tf_default |
+			mysql_ti_sql_type_info::tf_null |
+			mysql_ti_sql_type_info::tf_unsigned),
 	sql_type_info("MEDIUMINT NULL", typeid(Null<sql_mediumint>),
 			MYSQL_TYPE_INT24, mysql_ti_sql_type_info::tf_null),
 	sql_type_info("MEDIUMINT UNSIGNED NULL", typeid(Null<sql_mediumint_unsigned>), 
@@ -252,7 +282,7 @@ bool mysql_type_info::quote_q() const
 
 bool mysql_type_info::escape_q() const
 {
-	const type_info& ti = c_type();
+	const type_info& ti = base_type().c_type();
 	return ti == typeid(string) ||
 			ti == typeid(sql_enum) ||
 			ti == typeid(sql_blob) ||
